<?php

/*
 *  read config-files in INI-style
 *
 *  used here to parse language files
 *
 *  Ex: 
 *      [Section]
 *      key = value
 *
 *  Authors:
 *    Daniel Lorch <daniel@lorch.cc>
 */

function config_parse($lines) {
  /* Default section */
  $section = 'Default';

  for($i=0; $i<count($lines); ++$i) {
    /* Section */
    if(preg_match("'\s*\[([^\]]+)\]\s*'i" , $lines[$i], $matches))
      $section = $matches[1];
    /* Entry */
    else if(preg_match("/\s*(\S+)\s*=\s*(.*)\s*(?:#.*)?/", $lines[$i], $matches))
      $result[$section][$matches[1]] = $matches[2];
  }
  
  return $result;
}

/*
 *  Return most preferred language by the browser
 *
 *  $available is an array of available languages to your application
 *
 *  Authors:
 *    Daniel Lorch  
 */

function preferred_language($available)
{
  $list=explode(",", $_SERVER['HTTP_ACCEPT_LANGUAGE']);

  for($i=0; $i<count($list); ++$i)
  {
    if(strstr($list[$i], ';') === false) {
      // default value
      $quality = 1;
      $lang = $list[$i];
    } else {
      list($lang,$quality)=explode(";", $list[$i]);
      list(,$quality)=explode("=",$quality);  // remove "q="
    }

    $languages[$quality]=$lang;
  }

  krsort($languages);
  reset($languages); 

  foreach($languages as $quality => $language)
  {
    // remove dialect
    if(strstr($language, '-') !== false) {
      list($language,$dialect)=explode("-",$language);
    }
    
    // find first language that is available
    for($j=0; $j<count($available); ++$j)
      if($available[$j]==$lang) {
         return $lang;
      }
  }

  return false;
}

?>
