<?php

/*
 *  Perform an action
 *
 *  Authors:
 *    Daniel Lorch <daniel@lorch.cc>
 *
 */ 

include "auth.php";

include_once(dirname(__FILE__) . '/vm/vm.php');
include_once(dirname(__FILE__) . '/inc/common.php');
include_once(dirname(__FILE__) . '/inc/util.php');
include_once(dirname(__FILE__) . '/config.php');

/* vmailmgr interface */

$vm = new my_vmailmgr($_SESSION['domain'], rc4_decrypt($salt, $_SESSION['password']));

/* This is needed when the user presses the back button (if an error occured),
   because the browser does not keep the form values, as the page is defined
   not to be cached (because we are using sessions). */

if(isset($_POST['key'])) {
  $key = &$_POST['key'];

  $_SESSION['stored_forms'][$key] = $_POST['form'];

  if(ini_get("magic_quotes_gpc") && count($_SESSION['stored_forms'][$key]) > 0) {
    foreach($_SESSION['stored_forms'][$key] as $name => $value) {
      $_SESSION['stored_forms'][$key][$name] = stripslashes($value);
    }
  }
}
 
/* Create error page and halt further execution*/

function raise_error($string) {
  global $parts, $language;

  echo template_replace($parts['header'], array(
    'title'     => $language['error']['title'],
    'character' => $language['language']['encoding']
  ));
  
  echo template_replace($parts['error_header'], array(
    'header' => $language['error']['header']
  ));

  echo template_replace($parts['error_message'], array(
    'message' => $string,
    'back'    => $language['error']['back']
  ));
  
  echo $parts['error_footer'];
  echo $parts['footer'];

  exit;
}

/* Clean up form data */

if(count($_POST['form']) > 0) {
  foreach($_POST['form'] as $name => $value) {

    if(ini_get("magic_quotes_gpc")) {
      $_POST['form'][$name] = stripslashes($value);
    }

    switch($name) {
      case 'username':
      case 'personal':
        $_POST['form'][$name] = trim($value);
        break;

      case 'forwards':
        $_POST['form'][$name] = preg_split("/\s+/", $_POST['form'][$name]); 
        break;

      case 'softquota':
      case 'hardquota':
      case 'messagesize':
        if(preg_match("/\d+/", $value, $matches)) {
          $_POST['form'][$name] = $matches[0] * BYTE_UNIT;
        }
	else {
	  $_POST['form'][$name] = '';
	}
        break;

      case 'messagecount':
        if(preg_match("/\d+/", $value, $matches)) {
          $_POST['form'][$name] = $matches[0];
        }
	else {
	  $_POST['form'][$name] = '';
	}
        break;

      case 'expiry_year':
        if(preg_match("/\d{4}/", $value, $matches)) {
	  $_POST['form'][$name] = $matches[0];
	}
        break;
      
      case 'expiry_month':
      case 'expiry_day':
        if(preg_match("/\d{2}/", $value, $matches)) {
	  $_POST['form'][$name] = $matches[0];
	}
        break;
    }

  }

  if(!isset($_POST['form']['autoresponse_enabled'])) {
    $_POST['form']['autoresponse_enabled'] = '';
  }
}

// no date set (for example from the user page)
if(!isset($_POST['form']['expiry_month']) || !isset($_POST['form']['expiry_day']) || !isset($_POST['form']['expiry_month'])) {
  $_POST['form']['expiry'] = '';
}
// no date set or user wants to delete expiration
else if($_POST['form']['expiry_month'] == '' && $_POST['form']['expiry_day'] == '' && $_POST['form']['expiry_year'] == '') {
  $_POST['form']['expiry'] = '';
} else {
  $expiry = mktime(0, 0, 0, (int)$_POST['form']['expiry_month'], (int)$_POST['form']['expiry_day'], (int)$_POST['form']['expiry_year']);

  // is this a valid date?
  if($expiry == -1) {
    $_POST['form']['expiry'] = '';
  } else {
    $_POST['form']['expiry'] = $expiry;
  }
}

/* Decide what to do */

if(isset($_GET['action']) && $_GET['action'] != '') {
  $action = $_GET['action'];
} else if(isset($_POST['action']) && $_POST['action'] != '') {
  $action = $_POST['action'];
} else {
  $action = '';
}

switch($action) {

  case 'add_account':

    /* if setting an attribute fails (set_forwards() is such a case), then
       the user is sent back to the form, but the account was already created
       and will cause an 'virtual user already exists' the next time the user
       presses "add". this ensures that the account will be deleted if an
       error occurs - but only if it hadn't existed already before. */
    $account_exists = ($vm->user_info($_POST['form']['username']) !== false);

    if(trim($_POST['form']['username'] == ''))
      raise_error($language['error']['no_username']);
    else if(trim($_POST['form']['password']) == '')
      raise_error($language['error']['no_password']);
    else if($_POST['form']['password'] != $_POST['form']['password_repeat'])
      raise_error($language['error']['password_mismatch']);
    else if(!($vm->add_account($_POST['form']['username'], $_POST['form']['password'])
      && $vm->set_personal($_POST['form']['username'],     $_POST['form']['personal'])
      && $vm->set_softquota($_POST['form']['username'],    $_POST['form']['softquota'])
      && $vm->set_hardquota($_POST['form']['username'],    $_POST['form']['hardquota'])
      && $vm->set_messagesize($_POST['form']['username'],  $_POST['form']['messagesize'])
      && $vm->set_messagecount($_POST['form']['username'], $_POST['form']['messagecount'])
      && $vm->set_expiry($_POST['form']['username'],       $_POST['form']['expiry'])
      && $vm->set_forwards($_POST['form']['username'],     $_POST['form']['forwards']))) {

      /* otherwise the user would see the "user deleted" message from below */
      $error = $vm->last_response();

      if(!$account_exists) {
        $vm->delete_user($_POST['form']['username']);
      }
      
      raise_error($error);
    }
    break;

  case 'add_alias':

    /* this is explained at 'add_account' */
    $account_exists = ($vm->user_info($_POST['form']['username']) !== false);
    
    if(trim($_POST['form']['username'] == ''))
      raise_error($language['error']['no_username']);
    else if($_POST['form']['password'] != $_POST['form']['password_repeat'])
      raise_error($language['error']['password_mismatch']);
    else if(!($vm->add_alias($_POST['form']['username'],   $_POST['form']['password'])
      && $vm->set_personal($_POST['form']['username'],     $_POST['form']['personal'])
      && $vm->set_expiry($_POST['form']['username'],       $_POST['form']['expiry'])
      && $vm->set_forwards($_POST['form']['username'],     $_POST['form']['forwards']))) {

      $error = $vm->last_response();

      if(!$account_exists) {
        $vm->delete_user($_POST['form']['username']);
      }
      
      raise_error($error);
    }

    break;

  case 'edit_alias':

    if($_POST['form']['password'] != '' || $_POST['form']['password_repeat'] != '') {
      if($_POST['form']['password'] != $_POST['form']['password_repeat']) {
        raise_error($language['error']['password_mismatch']);
      }
      else if(!$vm->set_password($_POST['form']['username'], $_POST['form']['password'])) {
        raise_error($vm->last_response());
      }
    }
    
    if(!($vm->set_personal($_POST['form']['username'],             $_POST['form']['personal'])
      && $vm->set_expiry($_POST['form']['username'],               $_POST['form']['expiry'])
      && $vm->set_enabled($_POST['form']['username'],              $_POST['form']['account_enabled'])
      && $vm->set_forwards($_POST['form']['username'],             $_POST['form']['forwards'])
      )) {
      
      raise_error($vm->last_response());
    }
    break;

  case 'edit_account':

    if($_POST['form']['password'] != '' || $_POST['form']['password_repeat'] != '') {
      if($_POST['form']['password'] != $_POST['form']['password_repeat']) {
        raise_error($language['error']['password_mismatch']);
      }
      else if(!$vm->set_password($_POST['form']['username'], $_POST['form']['password'])) {
        raise_error($vm->last_response());
      }
    }
    
    if(!($vm->set_personal($_POST['form']['username'],             $_POST['form']['personal'])
      && $vm->set_softquota($_POST['form']['username'],            $_POST['form']['softquota'])
      && $vm->set_hardquota($_POST['form']['username'],            $_POST['form']['hardquota'])
      && $vm->set_messagesize($_POST['form']['username'],          $_POST['form']['messagesize'])
      && $vm->set_messagecount($_POST['form']['username'],         $_POST['form']['messagecount'])
      && $vm->set_expiry($_POST['form']['username'],               $_POST['form']['expiry'])
      && $vm->set_enabled($_POST['form']['username'],              $_POST['form']['account_enabled'])
      && $vm->set_forwards($_POST['form']['username'],             $_POST['form']['forwards'])
      && $vm->autoresponse_set($_POST['form']['username'],         'Subject: ' . $_POST['form']['autoresponse_subject'] . "\nFrom: " . $_POST['form']['username'] . "@" . $_SESSION['domain'] . "\n\n" . $_POST['form']['autoresponse_text'])
      && $vm->autoresponse_set_enabled($_POST['form']['username'], $_POST['form']['autoresponse_enabled'] != '')
      )) {
      
      raise_error($vm->last_response());
    }
    break;

  case 'remove':
    if(!$vm->delete_user($_GET['username'])) {
      raise_error($vm->last_response());
    }
    break;

  case 'catchall':
    if(!$vm->set_catchall($_POST['form']['catchall'])) {
      raise_error($vm->last_response());
    }  
    break;

  case 'user_alias':

    if($_POST['form']['password'] != '' || $_POST['form']['password_repeat'] != '') {
      if($_POST['form']['password'] != $_POST['form']['password_repeat']) {
        raise_error($language['error']['password_mismatch']);
      }
      else {
      
        if($vm->set_password($_SESSION['user'], $_POST['form']['password'])) {
          $vm->pass = $_POST['form']['password'];
          $_SESSION['password'] = rc4_encrypt($salt, $_POST['form']['password']);
        }
        else {
          raise_error($vm->last_response());
        }
	
      }
    }

    if(!($vm->set_forwards($_SESSION['user'], $_POST['form']['forwards'])
      )) {
                  
      raise_error($vm->last_response());
    }

    /* action successful - we don't need this anymore */
    if(isset($_POST['key'])) {
      unset($_POST['form'][$_POST['key']]);
    }

    Header('Location: user.php?key=' . $_POST['key'] . '&' . SID);
    exit;
    
    break;

  case 'user_account':

    if($_POST['form']['password'] != '' || $_POST['form']['password_repeat'] != '') {
      if($_POST['form']['password'] != $_POST['form']['password_repeat']) {
        raise_error($language['error']['password_mismatch']);
      }
      else {
      
        if($vm->set_password($_SESSION['user'], $_POST['form']['password'])) {
          $vm->pass = $_POST['form']['password'];
          $_SESSION['password'] = rc4_encrypt($salt, $_POST['form']['password']);
        }
        else {
          raise_error($vm->last_response());
        }
	
      }
    }

    if(!($vm->set_forwards($_SESSION['user'], $_POST['form']['forwards'])
      && $vm->autoresponse_set($_SESSION['user'], 'Subject: ' . $_POST['form']['autoresponse_subject'] . "\nFrom: " . $_SESSION['user'] . "@" . $_SESSION['domain'] . "\n\n" . $_POST['form']['autoresponse_text'])
      && $vm->autoresponse_set_enabled($_SESSION['user'], $_POST['form']['autoresponse_enabled'] != '')
      )) {
                  
      raise_error($vm->last_response());
    }

    /* action successful - we don't need this anymore */
    if(isset($_POST['key'])) {
      unset($_POST['form'][$_POST['key']]);
    }

    Header('Location: user.php?key=' . $_POST['key'] . '&' . SID);
    exit;
    
    break;

  case 'settings':
    $form = $_POST['form'];

    $form['forwards'] = @join("\n", $form['forwards']);
    
    if($form['softquota']    != '') $form['softquota'] /= BYTE_UNIT;
    if($form['hardquota']    != '') $form['softquota'] /= BYTE_UNIT;
    if($form['messagesize']  != '') $form['softquota'] /= BYTE_UNIT;

    setcookie("form", serialize($form));
    break;
}    

/* action successful - we don't need this anymore */
if(isset($_POST['key'])) {
  unset($_SESSION['stored_forms'][$_POST['key']]);
}

/* beam me back, scotty */
header("Location: main.php?" . SID);

?>
